/**
 * Copyright (c) 2013-present, Facebook, Inc.
 *
 * This source code is licensed under the MIT license found in the
 * LICENSE file in the root directory of this source tree.
 */

'use strict';

var createReactNativeComponentClass = require('../createReactNativeComponentClass')
  .default;

var View = createReactNativeComponentClass({
  validAttributes: {},
  uiViewClassName: 'View',
});

module.exports = View;
